function dS = vehicle_plant(t,S,U,P)
% A function to compute the time-derivatives of the states.
% =====Input=====
% t         double          current time of the simulation
% S         struct          The state vector.
%                           
% U         struct          The inpute vector.
% P         struct          Parameters of the vehicle and simulation.
% =====Output=====
% dS        vector          A vector representing the time derivatives of
%                           the states.
%% Get the states from the state vector S
x_dot = S(1);
y_dot = S(2);
psi = S(3);
psi_dot = S(4);
w_fl = S(5);
w_fr = S(6);
w_rl = S(7);
w_rr = S(8);
X = S(9);
Y = S(10);

%% Get parameters from P
m = P.m;         %Car mass;
I_z = P.I_z;     %yaw moment of inertia
l_f = P.l_f;
l_r = P.l_r;
l_w = P.l_w;
J_w = P.J_w ;    %Rotational moment of inertia of tire
r_eff = P.r_eff;
%%%%%%%delta = P.delta_profile(t);

%% Get input from U
T_dfl = U.T_dfl;
T_dfr = U.T_dfr;
T_drl = U.T_drl;
T_drr = U.T_drr;
T_bfl = U.T_bfl;
T_bfr = U.T_bfr;
T_brl = U.T_brl;
T_brr = U.T_brr;
delta = U.delta;
x_dot_des = U.x_dot_des;

%% Get tire forces from tire force model
tireForces = getTireForces_dugoff(t,S,P,delta);
F_xfl = tireForces.F_xfl;
F_xfr = tireForces.F_xfr;
F_xrl = tireForces.F_xrl;
F_xrr = tireForces.F_xrr;
F_yfl = tireForces.F_yfl;
F_yfr = tireForces.F_yfr;
F_yrl = tireForces.F_yrl;
F_yrr = tireForces.F_yrr;

%% Prob. 1.2a
%Compute x_ddot, y_ddot, and psi_ddot
x_ddot = ((F_xfl+F_xfr)*cos(delta)+F_xrl+F_xrr ...
          - (F_yfl+F_yfr)*sin(delta))/m + psi_dot*y_dot;
y_ddot = (F_yrl+F_yrr+(F_xfl+F_xfr)*sin(delta) + ...
          (F_yfl+F_yfr)*cos(delta))/m - psi_dot*x_dot;
psi_ddot = (l_f*(F_xfl+F_xfr)*sin(delta) + l_f*(F_yfl+F_yfr)*cos(delta) ...
            - l_r*(F_yrl+F_yrr) + l_w/2*(F_xrr-F_xrl) ...
            + l_w/2*(F_xfr-F_xfl)*cos(delta) ...
            + l_w/2*(F_yfl-F_yfr)*sin(delta))/I_z;

%% Prob. 1.2b
%Compute omega_dot for each wheel
w_fl_dot = (T_dfl-T_bfl-r_eff*F_xfl)/J_w;
w_fr_dot = (T_dfr-T_bfr-r_eff*F_xfr)/J_w;
w_rl_dot = (T_drl-T_brl-r_eff*F_xrl)/J_w;
w_rr_dot = (T_drr-T_brr-r_eff*F_xrr)/J_w;

%% Prob. 1.2c
%% Compute X_dot, Y_dot
X_dot = x_dot*cos(psi) - y_dot*sin(psi);
Y_dot = x_dot*sin(psi) + y_dot*cos(psi);

x_dot_err = x_dot_des-x_dot;

%% Orgainize output
dS = [x_ddot, y_ddot, psi_dot, psi_ddot, w_fl_dot, w_fr_dot, w_rl_dot, w_rr_dot, X_dot, Y_dot, x_dot_err]';
end